# frozen_string_literal: true

# Internal: Raised when an entry is not found in the build manifest.
#
# NOTE: The complexity here is justified by the improved usability of providing
# a more specific error message depending on the situation.
class ViteRuby::MissingEntrypointError < ViteRuby::Error
  extend Forwardable
  def_delegators :@info, :file_name, :last_build, :manifest, :config

  def initialize(info)
    @info = info
    super <<~MSG
      Vite Ruby can't find #{ file_name } in #{ config.manifest_path.relative_path_from(config.root) } or #{ config.assets_manifest_path.relative_path_from(config.root) }.

      Possible causes:
      #{ possible_causes(last_build) }
      :troubleshooting:
      #{ "\nContent in your manifests:\n#{ JSON.pretty_generate(manifest) }\n" if last_build.success }
      #{ "Last build in #{ config.mode } mode:\n#{ last_build.to_json }\n" unless last_build.success.nil? }
    MSG
  end

  def possible_causes(last_build)
    return FAILED_BUILD_CAUSES.sub(':mode:', config.mode) if last_build.success == false
    return DEFAULT_CAUSES if config.auto_build

    DEFAULT_CAUSES + NO_AUTO_BUILD_CAUSES
  end

  FAILED_BUILD_CAUSES = <<-MSG
  - The last build failed. Try running `bin/vite build --clear --mode=:mode:` manually and check for errors.
  MSG

  DEFAULT_CAUSES = <<-MSG
  - The file path is incorrect.
  - The file is not in the entrypoints directory.
  - Some files are outside the sourceCodeDir, and have not been added to watchAdditionalPaths.
  MSG

  NO_AUTO_BUILD_CAUSES = <<-MSG
  - You have not run `bin/vite dev` to start Vite, or the dev server is not reachable.
  - "autoBuild" is set to `false` in your config/vite.json for this environment.
  MSG
end
